/*
 * Created on Apr 11, 2005
 *
 */
package com.sunlabs.archsim.fleet.HornAndFunnel;

import java.util.Iterator;

import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.globals.GlobalDefaults;

/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 * @author ib27688
 *
 * SourceSync is a part of the source funnel - 
 * a component that takes in the
 * token from the instruction horn and data from a ship.
 * This is the action that is at the core of source sync.
 */
public class SourceSyncAction extends Action {

	
	Function passData;
	Function ackSrc;
	Function ackToken;
	
	public SourceSyncAction(String name) {
		super(name);
		passData= new PassData();
		ackToken= new Ack();
		ackSrc= new Ack();
		outputFunctions.put(FleetGlobal.OUT, passData);
		outputFunctions.put(FleetGlobal.ACK_IN, ackToken);
		outputFunctions.put(FleetGlobal.ACK_SRC_IN, ackSrc);
	}
	
	public SourceSyncAction(String name, CompositeEntity g) {
		this(name);
		g.addMember(this);
	}
	
	private class PassData extends Function {
		public PassData() {
			super("PassData");
			Integer fDelay= globals.intValue(GlobalDefaults.SOURCE_SYNC_PASS_DATA_DELAY);
			functionDelay= (fDelay != null)
				? new Delay(fDelay.intValue())
				: GlobalDefaults.SOURCE_SYNC_PASS_DATA_DELAY_DEF;
		}
		
		public Object value() {
			Iterator ti= terminals.iterator();
			Terminal t= null;
			boolean foundIn= false;
			boolean foundSrcIn= false;
			Payload payload= null;
			Data data= null;
			while ((!foundIn || !foundSrcIn) && ti.hasNext()) {
				t= (Terminal)(ti.next());
				if (t.getBaseName().equals(FleetGlobal.IN)) {
					foundIn= true;
					payload= (Payload)t.getData();
				}
				else if (t.getBaseName().equals(FleetGlobal.SRC_IN)) {
					foundSrcIn= true;
					data= (Data)t.getData();
				}
			}
			if (foundIn && foundSrcIn) {
				payload.data= data;
				return payload;
			}			
			else {
				return null;
			}
		}
	} // passData
	
	/** Vacuous function used to acknowldge that the action has fired */ 
	private class Ack extends Function {
		
		public Ack() {
			super("Vacous");
			Integer fDelay= globals.intValue(GlobalDefaults.SOURCE_SYNC_ACK_DELAY);
			functionDelay= (fDelay != null)
				? new Delay(fDelay.intValue())
				: GlobalDefaults.SOURCE_SYNC_ACK_DELAY_DEF;		
		}
	
		public Object value() {
			return new Data();
		} // value
	} // Ack
	
} // SourceSyncAction
